searchNodes=[{"ref":"ExDhcp.html","title":"ExDhcp","type":"behaviour","doc":"Creates an OTP-compliant DHCP server. An ExDhcp module binds to a UDP port, listens to DHCP messages sent to the port, and can resend DHCP messages (typically as a broadcast message) in response. You may instrument whatever state you would like into the server; the DHCP-specific contents will be encapsulated into the internals of the server itself and all of the state exposed to the callbacks will be your own state. ExDhcp is not a fully-functional DHCP server which conforms to the RFC 1531 specs. It does not: keep the state of the DHCP leases store DHCP lease information to durable storage manage arp tables However, you could use ExDhcp to implement that functionality. For example, minimal ExDhcp server might look something like this: defmodule MyDhcpServer do use ExDhcp alias ExDhcp.Packet def start_link(init_state) do ExDhcp.start_link(__MODULE__, init_state) end @impl true def init(init_state), do: {:ok, init_state} @impl true def handle_discover(request, xid, mac, state) do # insert code here. Should assign the unimplemented values # for the response below: response = Packet.respond(request, :offer, yiaddr: issued_your_address, siaddr: server_ip_address, subnet_mask: subnet_mask, routers: [router], lease_time: lease_time, server: server_ip_address, domain_name_servers: [dns_server])) {:respond, response, new_state} end @impl true def handle_request(request, xid, mac, state) do # insert code here. response = Packet.respond(request, :ack, yiaddr: issued_your_address ...) {:respond, response, state} end @impl true def handle_decline(request, xid, mac, state) do # insert code here. response = Packet.respond(request, :offer, yiaddr: new_issued_address ...) {:respond, response, state} end end You must implement the following three DHCP functionalities as callbacks to successfully assign network hosts: handle_discover/4 handle_request/4 handle_decline/4 You may also want to implement these callbacks: handle_inform/4 to manage a DHCP client requesting early info handle_release/4 to manage DHCP clients relinquishing their leases handle_packet/4 to generically handle DHCP packets this is most useful to monitor how other DHCP servers are responding on your network Each of these DHCP callbacks take four arguments, in order: the fully-parsed ExDhcp.Packet structure, the xid (transaction id) the mac address of the client the current state of the ExDhcp GenServer These are provided as arguments to enable you to easily trap their contents in pattern-matching guards. NB: the ExDhcp.Packet structure has an :options field, and you may want to emit tags that are outside of the provided basic dhcp options (see ExDhcp.Options.Basic). In this case, you should implement your own parser module (see ExDhcp.Options.Macro) and instrument it into your ExDhcp module using the :dhcp_options option like so: use Dhcp, dhcp_options: [ExDhcp.Options.Basic, YourParserModule, AnotherParserModule] ExDhcp also provides these optional callbacks handle_call/3 handle_cast/2 handle_continue/2 handle_info/2 These operate identically to their counterparts in GenServer, but note that they are passed their encapsulated state, not the raw state of the GenServer."},{"ref":"ExDhcp.html#child_spec/1","title":"ExDhcp.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"ExDhcp.html#c:handle_call/3","title":"ExDhcp.handle_call/3","type":"callback","doc":"See GenServer.handle_call/3"},{"ref":"ExDhcp.html#c:handle_cast/2","title":"ExDhcp.handle_cast/2","type":"callback","doc":"See GenServer.handle_cast/2"},{"ref":"ExDhcp.html#c:handle_continue/2","title":"ExDhcp.handle_continue/2","type":"callback","doc":"See GenServer.handle_continue/2"},{"ref":"ExDhcp.html#c:handle_decline/4","title":"ExDhcp.handle_decline/4","type":"callback","doc":"Responds to the DHCP decline query, as encoded in option 53."},{"ref":"ExDhcp.html#c:handle_discover/4","title":"ExDhcp.handle_discover/4","type":"callback","doc":"Responds to the DHCP discover query, as encoded in option 53."},{"ref":"ExDhcp.html#c:handle_info/2","title":"ExDhcp.handle_info/2","type":"callback","doc":"See GenServer.handle_info/2"},{"ref":"ExDhcp.html#c:handle_inform/4","title":"ExDhcp.handle_inform/4","type":"callback","doc":"Responds to the DHCP inform query, as encoded in option 53. Defaults to ignore."},{"ref":"ExDhcp.html#c:handle_packet/4","title":"ExDhcp.handle_packet/4","type":"callback","doc":"Responds to other DHCP queries or broadcast packet types your typical server may not be setup to listen to. Leader contention or race conditions may arise in certain situations. For example, a DHCP request might have been handled by another server already and broadcasted over the layer 2 network. To avoid these issues, your server may want to act on its internal state based on the information transmitted in these packets. Use this callback to implement these features. Server queries you might also want to monitor are: DHCP_OFFER (2) DHCP_ACK (5) DHCP_NAK (6) You should also use a custom handle_packet routine if you override ExDhcp.Options.Basic with your own DHCP options parser that overwrites option 53 with a different atom/value assignment scheme."},{"ref":"ExDhcp.html#c:handle_release/4","title":"ExDhcp.handle_release/4","type":"callback","doc":"Responds to the DHCP release query, as encoded in option 53. Defaults to ignore."},{"ref":"ExDhcp.html#c:handle_request/4","title":"ExDhcp.handle_request/4","type":"callback","doc":"Responds to the DHCP request query, as encoded in option 53."},{"ref":"ExDhcp.html#c:init/1","title":"ExDhcp.init/1","type":"callback","doc":"Invoked on the new DHCP server process when started by ExDhcp.start_link/3 Will typically emit {:ok, state}, where state is the initial state you expect to be contained within your ExDhcp GenServer."},{"ref":"ExDhcp.html#start_link/3","title":"ExDhcp.start_link/3","type":"function","doc":"Caller function initiating the spawning of your ExDhcp GenServer. You may supply the following extra options: :port the UDP port you&#39;d like ExDhcp to listen in on; defaults to 6767. you may want to change this to 67 if you do not want to use iptables to redirect DHCP transactions to a nonprivileged Elixir server. :bind_to_device (must be a binary string), the device you would like to bind to for DHCP listening. This is most useful when you have a device with a internal-and -external-facing net interfaces and you would like to only respond to DHCP requests coming from select directions. This requires cap_net_raw to be set. In Linux systems, this is settable as superuser using the following command: setcap cap_net_raw=ep /path/to/beam.smp :client_port specify a nonstandard port to send the response to. Most useful for testing purposes; defaults to 68. :broadcast_addr to specify a nonstandard address for responses. Defaults to {255, 255, 255, 255}. See GenServer.start_link/3 for further options."},{"ref":"ExDhcp.html#c:terminate/2","title":"ExDhcp.terminate/2","type":"callback","doc":"See GenServer.terminate/2"},{"ref":"ExDhcp.html#t:response/0","title":"ExDhcp.response/0","type":"type","doc":"DHCP callbacks should provide either a :respond or :norespond outcome. In the case of the :respond outcome, a UDP response is sent over the open port earmarked for the specified mac address. The :norespond outcome is a no-op. No information is sent over the wire, and the client may choose to either continue sending requests on the presumption that the UDP packets were dropped, or initiate an entirely new transaction."},{"ref":"ExDhcp.Options.Basic.html","title":"ExDhcp.Options.Basic","type":"module","doc":"This module implements the following DHCP options encodings: option atomDHCP option codetype / codec :subnet_mask1ip :routers3iplist :domain_name_servers6iplist :host_name12string :broadcast_address28ip :requested_address50ip :lease_time51integer :message_type53encode_message_type/1 decode_message_type/1 :server54ip :parameter_request_list55encode_parameter_list/1 decode_parameter_list/1 :message56string :max_message_size57integer :renewal_time58integer :rebinding_time59integer :vendor_class_identifier60string :client_identifier61string :client_system93string :client_ndi94encode_client_ndi/1 decode_client_ndi/1 :uuid_guid97uuid :forcerenew_nonce_capable145boolean"},{"ref":"ExDhcp.Options.Basic.html#decode_client_ndi/1","title":"ExDhcp.Options.Basic.decode_client_ndi/1","type":"function","doc":"Decodes the client_ndi information (code 94) from the DHCP binary format into an erlang triple."},{"ref":"ExDhcp.Options.Basic.html#decode_message_type/1","title":"ExDhcp.Options.Basic.decode_message_type/1","type":"function","doc":"Decodes the message_type (code 53) field from a DHCP packet. Translates the value according to the following table: dhcp codeatom 1:discover 2:offer 3:request 4:decline 5:ack 6:nak 7:release 8:inform"},{"ref":"ExDhcp.Options.Basic.html#decode_parameter_list/1","title":"ExDhcp.Options.Basic.decode_parameter_list/1","type":"function","doc":"Decodes the parameter_request_list (code 55) from a DHCP packet. Returns a list of integers that represent parameters that the DHCP client is currently requesting from the DHCP server. The server is not required to respond to them, though if key parameters are missing, the client might send a DHCP decline response."},{"ref":"ExDhcp.Options.Basic.html#encode_client_ndi/1","title":"ExDhcp.Options.Basic.encode_client_ndi/1","type":"function","doc":"Encodes the client_ndi information (code 94) from an erlang triple into the DHCP binary format."},{"ref":"ExDhcp.Options.Basic.html#encode_message_type/1","title":"ExDhcp.Options.Basic.encode_message_type/1","type":"function","doc":"Encodes the message_type (code 53) field atom back into a DHCP binary format. See decode_message_type/1 for the translation table."},{"ref":"ExDhcp.Options.Basic.html#encode_parameter_list/1","title":"ExDhcp.Options.Basic.encode_parameter_list/1","type":"function","doc":"Encodes the parameter_request_list (code 55) into the DHCP binary format. See decode_parameter_list/1 for details."},{"ref":"ExDhcp.Options.Basic.html#t:client_ndi_type/0","title":"ExDhcp.Options.Basic.client_ndi_type/0","type":"type","doc":"Erlang triple format for client_ndi information."},{"ref":"ExDhcp.Options.Basic.html#t:message_types/0","title":"ExDhcp.Options.Basic.message_types/0","type":"type","doc":"DHCP messages must fall into one of these listed categories."},{"ref":"ExDhcp.Options.Macro.html","title":"ExDhcp.Options.Macro","type":"module","doc":"This module provides methods that facilitate custom encoding and decoding strategies for DHCP packet options. ExDhcp.Options.Basic provides basic parameter encoding; the full DHCP specification provides for additional, proprietary, and custom options encoding. For example, PXE (Preboot eXecution Environment) uses an additional set of options to transmit booting information to the client. These are implemented in ExDhcp.Options.Pxe for demonstration. If you need to implement additional options parsing use this module, include it as an option in the use ExDhcp directive along with ExDhcp.Options.Basic (unless you want to override it): use ExDhcp, dhcp_options: [MyParser, ExDhcp.Options.Basic] Standard and Custom Parsing Pass the options/1 macro a keyword list of parameter names and datatypes. The options parser will then search for a module parameter corresponding to the keyword, and assign that integer as the options parameter. The type will indicate either a standard type codec or an atom for a custom codec. The following standard types are implemented: NameErlang Term TypeBinary Representation :ip{a, b, c, d}4 octets :iplistlist({a, b, c, d})Nx4 octets :stringbinaryvariable octets :uuid&lt;&lt;::binary-size(36)&gt;&gt;16 octets :integerinteger4 octet (32 bit) integer :shortinteger2 octet (16 bit) integer :byteinteger1 octet (8 bit) integer :booleanbooleanone octet, either 1 or 0 In the case of a custom codec, you must implement two functions: encode_&lt;atom_value&gt;/1: &nbsp; The encoder should take a raw binary and convert it to an appropriate erlang term representing the atom. decode_&lt;atom_value&gt;/1: &nbsp; The decoder should take an erlang term and convert it to an appropriate binary to be packed into the DHCP packet. options/1 will append a relevant table of encoders/decoders into your module documentation as a feature. Here is an example implementation of a parser: defmodule MyParser do import ExDhcp.Options.Macro @behaviour ExDhcp.Options.Api @option_1 123 @option_2 124 @option_3 125 options option_1: :integer option_2: :string option_3: :option_3 def decode_option_3(binary) do # code_to_decode_option_3 result_erlang_term end def encode_option_3(source_erlang_term) do # code_to_encode_option_3 result_binary end end Refer to ExDhcp.Options.Basic and ExDhcp.Options.Pxe source codes as an additional reference. Learn more about PXE here: Wikipedia"},{"ref":"ExDhcp.Options.Macro.html#options/1","title":"ExDhcp.Options.Macro.options/1","type":"macro","doc":"Generates code for codecs based on a list of atom / type keys. See ExDhcp.Options.Macro for details and strategies for using this macro."},{"ref":"ExDhcp.Options.Pxe.html","title":"ExDhcp.Options.Pxe","type":"module","doc":"DHCP options for PXE (Preboot eXecution Environment) booting systems. NB: Learn more about PXE here: Wikipedia This module implements the following DHCP options encodings: option atomDHCP option codetype / codec :tftp_server_name66string :bootfile_name67string :tftp_server150ip :tftp_config209string :tftp_root210string"},{"ref":"ExDhcp.Packet.html","title":"ExDhcp.Packet","type":"module","doc":"Provides a structure for the DHCP UDP packet, according to RFC 1531 specifications. For a simpler reference on the DHCP protocol&#39;s binary layout, refer to Wikipedia. OP: operation (request: 1, response: 2). ExDhcp will only respond to requests (except in handle_packet) and only send response packets. HTYPE: specifies the hardware address type. Currently only ethernet is supported. HLEN: specifies the hardware address length. Currently only 6-byte MAC is supported. HOPS: number of hops. For when you implement &#39;relay-DHCP&#39; (see RFC 1531: BOOTP relay agent). XID: transaction id. Allows concurrent servicing of multiple DHCP requests. You may want to implement spawning of separate servers to handle different transmissions. SECS: seconds since client has booted. FLAGS: DHCP flags (see RFC 1531: figure 2). CIADDR: &quot;client internet address&quot; (expected in :request requests). YIADDR: &quot;your (client) internet address&quot; (expected in :offer responses) SIADDR: &quot;next server internet address&quot; (expected in some :offer, :ack, and :nak responses) GIADDR: &quot;gateway internet address&quot;. For when you implement &#39;relay-DHCP&#39; (see RFC 1531: BOOTP relay agent). options: a {integer, binary} tuple list. Supported opcodes will be translated into {atom, value} tuples by ExDhcp.Options parser modules (see ExDhcp.Options.Macro). Learn more about RFC 1531 here: ietf.org"},{"ref":"ExDhcp.Packet.html#decode/2","title":"ExDhcp.Packet.decode/2","type":"function","doc":"Converts a udp packet or a binary payload from a UDP packet and converts it to a ExDhcp.Packet struct. NB: This function will fail if you attempt to pass it a UDP packet that does not contain the DHCP &quot;magic cookie&quot;."},{"ref":"ExDhcp.Packet.html#encode/2","title":"ExDhcp.Packet.encode/2","type":"function","doc":"Converts from a ExDhcp.Packet struct into an iolist(). Typically, this will be sent directly to a :gen_udp.send/2 call. If you need to examine the contents of the iolist() as a binary, you may want to send the results to :erlang.iolist_to_binary/1"},{"ref":"ExDhcp.Packet.html#respond/3","title":"ExDhcp.Packet.respond/3","type":"function","doc":"A convenience function used to craft a DHCP response based on the request. Usage type should be one of [:offer, :ack, :nak]. The built-in values are copied into the response without change. The DHCP opcode is automatically set to 2. The options list is stripped. params should be a flat keyword list containing DHCP parameters and options. All of the options keys should be encodable by exactly one of your options parsing modules. If you need to encode a value directly as an integer/binary pair because it&#39;s not parsed by any modules, do not use respond/3. Example iex&gt; ExDhcp.Packet.respond(%ExDhcp.Packet{}, :offer, yiaddr: {192, 168, 0, 5}, hostname: &quot;foo&quot;) %ExDhcp.Packet{yiaddr: {192, 168, 0, 5}, options: %{53 =&gt; &lt;&lt;2&gt;&gt;, hostname: &quot;foo&quot;}} Note that in the example :yiaddr entered the packet struct, and :hostname entered the :options parameter."},{"ref":"ExDhcp.Packet.html#t:t/0","title":"ExDhcp.Packet.t/0","type":"type","doc":"The Packet struct type. See ExDhcp.Packet for details on the struct parameters."},{"ref":"ExDhcp.Packet.html#t:udp_packet/0","title":"ExDhcp.Packet.udp_packet/0","type":"type","doc":"Erlang&#39;s internal representation of an active UDP packet."},{"ref":"ExDhcp.Utils.html","title":"ExDhcp.Utils","type":"module","doc":"Provides utilities containing typespecs for data types and binary/string conversions for ip and mac addresses. For ease-of-use both within this library and when using it."},{"ref":"ExDhcp.Utils.html#bin2ip/1","title":"ExDhcp.Utils.bin2ip/1","type":"function","doc":"Converts a binary stored ip address (in the dhcp packet spec) to an erlang-style ip address. (4-tuple of bytes)"},{"ref":"ExDhcp.Utils.html#bin2mac/1","title":"ExDhcp.Utils.bin2mac/1","type":"function","doc":"Converts a binary stored mac address (in the dhcp packet spec) to an erlang-style mac address. (6-tuple of bytes)"},{"ref":"ExDhcp.Utils.html#cidr2mask/1","title":"ExDhcp.Utils.cidr2mask/1","type":"function","doc":"Creates a subnet mask from a cidr value. iex&gt; ExDhcp.Utils.cidr2mask(24) {255, 255, 255, 0}"},{"ref":"ExDhcp.Utils.html#ip2bin/1","title":"ExDhcp.Utils.ip2bin/1","type":"function","doc":"Converts an erlang-style ip address (4-tuple of bytes) to a binary stored ip address (in the dhcp packet spec)"},{"ref":"ExDhcp.Utils.html#ip2str/1","title":"ExDhcp.Utils.ip2str/1","type":"function","doc":"Represents an erlang-style ip4 value as a string, without going through a list intermediate."},{"ref":"ExDhcp.Utils.html#mac2bin/1","title":"ExDhcp.Utils.mac2bin/1","type":"function","doc":"Converts an erlang-style mac address (6-tuple of bytes) to a binary stored mac address (in the dhcp packet spec)."},{"ref":"ExDhcp.Utils.html#mac2str/1","title":"ExDhcp.Utils.mac2str/1","type":"function","doc":"Converts a mac address 6-byte tuple to a string. iex&gt; ExDhcp.Utils.mac2str({1, 2, 3, 16, 255, 254}) &quot;01:02:03:10:FF:FE&quot;"},{"ref":"ExDhcp.Utils.html#str2mac/1","title":"ExDhcp.Utils.str2mac/1","type":"function","doc":"Converts a mac address string into a raw binary value. iex&gt; ExDhcp.Utils.str2mac(&quot;01:02:03:10:FF:FE&quot;) {1, 2, 3, 16, 255, 254}"},{"ref":"ExDhcp.Utils.html#t:ip4/0","title":"ExDhcp.Utils.ip4/0","type":"type","doc":"Erlang-style ip addresses."},{"ref":"ExDhcp.Utils.html#t:mac/0","title":"ExDhcp.Utils.mac/0","type":"type","doc":"Mac addresses in the same style as the erlang ip address."}]